!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!   ace :    A nonlinear diffusion problem (The Allen­Cahn equation)
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
!     ---------- ---- 

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
      DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM), DFDP(NDIM,*)
      DOUBLE PRECISION lambda

       lambda=PAR(1)

!      Set the nonlinear term
       F(1) = lambda*U(1) + U(1)**3 - U(1)**5 

      END SUBROUTINE FUNC
!---------------------------------------------------------------------- 
      SUBROUTINE STPNT(NDIM,U,PAR,x) 
!     ---------- ----- 

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
      DOUBLE PRECISION, INTENT(IN) :: x
      DOUBLE PRECISION lambda, L, c, pi, amp

!      Set the parameters:
       lambda = 25.0d0

!      Set the actual width of the space interval [0,L]:
       L = 2.d0

!      Set the diffusion constant:
       c = 0.25d0

       PAR(1)  = lambda
       PAR(11) = L
       PAR(15) = c

!      Set the initial data in the (scaled) interval [0,1]:
       amp=0.1d0
       pi = 4*DATAN(1.d0)
       U(1)=amp*DSIN(pi*x)

!      The derivative of U(1) with respect to x must also be given.
!      Note the scaling by 1/L, due to the scaling of x to [0,1]:
       U(2)=amp*pi*DCOS(pi*x)/L  

      END SUBROUTINE STPNT
!---------------------------------------------------------------------- 

      SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC) 
!     ---------- ---- 

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
      DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
      DOUBLE PRECISION, INTENT(OUT) :: FB(NBC)
      DOUBLE PRECISION, INTENT(INOUT) :: DBC(NBC,*)

! The boundary conditions u(0)=0 and u(L)=0 are defined here for
! the scaled interval [0,1], i.e., u(0)=0 and u(1)=0.

       FB(1)=U0(1) 
       FB(2)=U1(1) 

      END SUBROUTINE BCND
!---------------------------------------------------------------------- 
      SUBROUTINE PVLS(NDIM,U,PAR)
!     ---------- ----

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NDIM
      DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
      DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)

      DOUBLE PRECISION, EXTERNAL :: GETP,GETU2
      INTEGER NDX,NCOL,NTST

! Set PAR(3) equal to the integral of U(1) (scaled to the interval [0,1])
       PAR(3)=GETP('INT',1,U)

      END SUBROUTINE PVLS
!---------------------------------------------------------------------- 
      SUBROUTINE ICND
      END SUBROUTINE ICND

      SUBROUTINE FOPT
      END SUBROUTINE FOPT
!---------------------------------------------------------------------- 
